//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.inventory.commands.*;

import java.rmi.*;
import java.sql.*;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.ejb.*;
import javax.naming.*;

import com.ibm.commerce.base.helpers.EJBConstants;
import com.ibm.commerce.command.*;
import com.ibm.commerce.copyright.IBMCopyright;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.entityadmin.EAConstants;
import com.ibm.commerce.entityadmin.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.inventory.commands.InventoryConstants;
import com.ibm.commerce.inventory.objects.AvailableReceiptAccessBean;
import com.ibm.commerce.inventory.objects.DistributionArrangementAccessBean;
import com.ibm.commerce.inventory.objects.ItemFulfillmentCenterAccessBean;
import com.ibm.commerce.inventory.objects.ItemSpecificationAccessBean;
import com.ibm.commerce.inventory.objects.InventoryJDBCHelperAccessBean;
import com.ibm.commerce.inventory.objects.VersionSpecificationAccessBean;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.scheduler.commands.*;
import com.ibm.commerce.server.ECConstants;
import com.ibm.commerce.tools.command.ToolsControllerCommandImpl;
import com.ibm.commerce.tools.common.ui.UIProperties;
import com.ibm.commerce.tools.common.*;
import com.ibm.commerce.tools.util.ResourceDirectory;

/**
 * Make a receipt available to the distribution arrangement.
 **/
public class MakeReceiptAvailableCmdImpl
	extends TaskCommandImpl
	implements MakeReceiptAvailableCmd 
{
	/**
	 * The internal copyright field.
	 */
	public static final String COPYRIGHT =
		com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

	private Long inDistArrangId = null;
	private Long inItemspcId = null;
	private Long inRcptAvailId = null;
	private Long inReceiptId = null;
	private TypedProperty ihshRequestProperties = null;

	/**
	 * Return distribution arrangement identifier.
	 * @return java.lang.Long
	 */
	public java.lang.Long getDistArrangId()
	{
		return inDistArrangId;
	}

	/**
	 * Return item identifier.
	 * @return java.lang.Long
	 */
	public java.lang.Long getItemspcId()
	{
		return inItemspcId;
	}
 
	/**
	 * Return available receipt identifier. 
	 * @return java.lang.Long
	 */
	public java.lang.Long getRcptAvailId()
	{
		return inRcptAvailId;
	}
	
	/**
	 * Return the receipt identifier.
	 * @return java.lang.Long
	 */
	public java.lang.Long getReceiptId()
	{
		return inReceiptId;
	}

	/**
	 * Set distribution arrangement identifier.
	 * @param newDistArrangId java.lang.Long
	 */
	public void setDistArrangId(java.lang.Long newDistArrangId)
	{
		inDistArrangId = newDistArrangId;
	}

 	/**
	 * Set item identifier.
	 * @param newItemspcId java.lang.Long
	 */
	public void setItemspcId(java.lang.Long newItemspcId)
	{
		inItemspcId = newItemspcId;
	}

 	/**
	 * Set available receipt identifier. 
	 * @param newRcptAvailId java.lang.Long
	 */
	public void setRcptAvailId(java.lang.Long newRcptAvailId)
	{
		inRcptAvailId = newRcptAvailId;
	}
	
 	/**
	 * Set the receipt identifier.
	 * @param newReceiptId java.lang.Long
	 */
	public void setReceiptId(java.lang.Long newReceiptId)
	{
		inReceiptId = newReceiptId;
	}
	 
	/**
	 * Set request properties.
	 * @param reqParms com.ibm.commerce.datatype.TypedProperty
	 */
	public void setRequestProperties(com.ibm.commerce.datatype.TypedProperty newRequestProperties)
		throws ECApplicationException
	{
		final String methodName = "setRequestProperties";
		ECTrace.entry(
			ECTraceIdentifiers.COMPONENT_INVENTORY,
			getClass().getName(),
			methodName);

		ihshRequestProperties = newRequestProperties;

 		setDistArrangId(ihshRequestProperties.getLong(InventoryConstants.DISTARRANG_ID, null));
 		setItemspcId(ihshRequestProperties.getLong(InventoryConstants.ITEMSPC_ID, null));
 		setReceiptId(ihshRequestProperties.getLong(InventoryConstants.RECEIPT_ID, null));
 
		ECTrace.exit(
			ECTraceIdentifiers.COMPONENT_INVENTORY,
			getClass().getName(),
			methodName);
	}
		
	/**
	 * This command is ready to be executed.
	 */			
 	public boolean isReadyToCallExecute()
	{
		return true;
	}

	/**
	 * This method checks the information needed for making a receipt available.
	 * 
	 * @exception com.ibm.commerce.exception.ECApplicationException. If mandatory
	 * parameters are not passed in, throw ECApplicationException.
	 */		
	public void validateParameters() throws ECException 
	{
		final String methodName = "validateParameters";
		ECTrace.entry(
			ECTraceIdentifiers.COMPONENT_INVENTORY,
			this.getClass().getName(),
			methodName);
  
		if (getReceiptId() == null)
		{
			throw new ECApplicationException(
				ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,
				getClass().getName(),
				methodName,
				ECMessageHelper.generateMsgParms(InventoryConstants.RECEIPT_ID));
		}
		if ((getDistArrangId() == null)
			&& (getItemspcId() == null))
		{
			throw new ECApplicationException(
				ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,
				getClass().getName(),
				methodName,
				ECMessageHelper.generateMsgParms(InventoryConstants.DISTARRANG_ID));
		}

		ECTrace.exit(
			ECTraceIdentifiers.COMPONENT_INVENTORY,
			this.getClass().getName(),
			methodName);
	}
	
	/**
	 * This method will make a receipt available to the distribution arrangement.
	 *
	 * <p>This is done by inserting a row into rcptavail table associating the receipt with 
	 * the distribution arrangement. 
	 *
	 * <p> The distribution arrangement will have a picking method of either LIFO or FIFO. If
	 * the picking method of the distribution arrangment is FIFO, the precedence of the rcptavail
	 * row will be MAX(precedence)+1. If the picking method of the distribution arrangment is 
	 * LIFO, the precedence of the rcptavail row will be MIN(precedence)-1. If there are no rows 
	 * in the rcptavail table, the precedence of the rcptavail row will be 1.
     *	 
	 * @exception com.ibm.commerce.exception.ECException
	 */
	public void performExecute() throws ECException 
	{
		final String methodName = "performExecute";
		ECTrace.entry(
			ECTraceIdentifiers.COMPONENT_INVENTORY,
			getClass().getName(),
			methodName);

		super.performExecute();

		java.sql.Timestamp currentTime =
			new java.sql.Timestamp(java.util.Calendar.getInstance().getTime().getTime());

		try
		{
			if (getDistArrangId() == null)
			{
				// Get baseitem_id from itemspc_id
				ItemSpecificationAccessBean abItemspc = new ItemSpecificationAccessBean();
				abItemspc.setInitKey_itemspcId(getItemspcId().toString());
				String strBaseItemId = abItemspc.getBaseItemId();

				// Get distarrang_id from baseitem_id
				java.util.Enumeration enumDistArrang =
					new DistributionArrangementAccessBean().findByWholeSaleStoreIdAndBaseItemId(
						getStoreId(),
						new Long(strBaseItemId));
				if (enumDistArrang.hasMoreElements())
				{
					DistributionArrangementAccessBean abDistArrang =
						(DistributionArrangementAccessBean) enumDistArrang.nextElement();
					setDistArrangId(new Long(abDistArrang.getDistArrangId()));
				}
			}
				
			if (getDistArrangId() == null)
			{
				// should give an error here as distArrangId cannot be null in rcptAvail table
				throw new ECApplicationException(
					ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,
					this.getClass().getName(),
					methodName,
					ECMessageHelper.generateMsgParms(InventoryConstants.DISTARRANG_ID));
			}

			// Get picking method from distribution arrangement
			String strPickingMethod = null;
 			DistributionArrangementAccessBean abDistArrang =
				new DistributionArrangementAccessBean();
			abDistArrang.setInitKey_distArrangId(getDistArrangId().toString());
			strPickingMethod = abDistArrang.getPickingMethod();

			// Get next precedence to use for new rcptavail row
			Integer nPrecedence = null;
			int nMaxPrecedence = 0;
			int nMinPrecedence = 0;
			int nNextPrecedence = 0;

			Vector vecRcptAvails = new Vector();
			vecRcptAvails =
				new InventoryJDBCHelperAccessBean().findAvailableReceiptMaxAndMinPrecedence();

			int n = vecRcptAvails.size();
			if (n > 0)
			{
				Vector rcptAvail = (Vector) vecRcptAvails.elementAt(0);
				Integer maxPrecedence = (Integer) rcptAvail.elementAt(0);
				if (maxPrecedence != null)
					nMaxPrecedence = maxPrecedence.intValue();
				Integer minPrecedence = (Integer) rcptAvail.elementAt(1);
				if (minPrecedence != null)
					nMinPrecedence = minPrecedence.intValue();
			}
			if (strPickingMethod != null && strPickingMethod.equals("F"))
			{
				nNextPrecedence = nMaxPrecedence + 1;
			}
			else
			{
				nNextPrecedence = nMinPrecedence - 1;
			}
			nPrecedence = new Integer(nNextPrecedence);

			// Create record in rcptavail table
			AvailableReceiptAccessBean abRcptavail =
				new AvailableReceiptAccessBean(getReceiptId(), getDistArrangId());
			abRcptavail.setPrecedence(nPrecedence);
			abRcptavail.setLastUpdate(currentTime.toString());
			abRcptavail.commitCopyHelper();
			setRcptAvailId(new Long(abRcptavail.getRcptAvailId()));
		}
		catch (CreateException e)
		{
			throw new ECSystemException(
				ECMessage._ERR_CREATE_EXCEPTION,
				this.getClass().getName(),
				methodName,
				ECMessageHelper.generateMsgParms(e.getMessage()),
				e);
		}
		catch (FinderException e)
		{
			throw new ECSystemException(
				ECMessage._ERR_FINDER_EXCEPTION,
				this.getClass().getName(),
				methodName,
				ECMessageHelper.generateMsgParms(e.getMessage()),
				e);
		}		
		catch (NamingException e)
		{
			throw new ECSystemException(
				ECMessage._ERR_NAMING_EXCEPTION,
				this.getClass().getName(),
				methodName,
				ECMessageHelper.generateMsgParms(e.getMessage()),
				e);
		}		
		catch (RemoteException e)
		{
			throw new ECSystemException(
				ECMessage._ERR_REMOTE_EXCEPTION,
				this.getClass().getName(),
				methodName,
				ECMessageHelper.generateMsgParms(e.getMessage()),
				e);
		}
		catch (SQLException e)
		{
			throw new ECSystemException(
				ECMessage._ERR_SQL_EXCEPTION,
				this.getClass().getName(),
				methodName,
				ECMessageHelper.generateMsgParms(e.getMessage()),
				e);
		}
			
		ECTrace.exit(
			ECTraceIdentifiers.COMPONENT_INVENTORY,
			getClass().getName(),
			methodName);
	}
	
	/**
	 * This method resets the command local variable.
	 */	
	public void reset()
	{
		super.reset();
	}
}