//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.messaging.commands.*;

import java.util.*;
import java.rmi.*;
import java.sql.Timestamp;
import java.math.BigDecimal;
import javax.ejb.*;
import javax.naming.*;
import java.lang.*;
import com.ibm.sfc.cmd.*;
import com.ibm.commerce.base.objects.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.order.utils.MiscCmd;
import com.ibm.commerce.common.objects.*;
import com.ibm.commerce.catalog.objects.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.user.objects.*;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.datatype.*;

import com.ibm.commerce.messaging.commandmapper.xml.*;
import com.ibm.commerce.orderstatus.objects.*;
import com.ibm.commerce.orderstatus.objimpl.*;
import com.ibm.commerce.messaging.util.*;
import com.ibm.commerce.accesscontrol.*;
/**
 * The implementation of the OrderStatus command (see interface for a detailed
 * description of the OrderStatus command)
 */
public class OrderStatusCmdImpl  extends ControllerCommandImpl implements OrderStatusCmd
{
	/**
	 * The internal copyright field.
	 */	
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

	
	protected TypedProperty osProp = null;
	protected Vector osItems = null;
//	protected boolean iResult = false;
	protected Long iOrderId = null;
	protected String iMerchantOrderNumber = null;

/**
 * Verifies that the sequence information of the order status to update does not
 * conflict with the sequence information with the order status that is currently in
 * the ORDSTAT table.  If the new sequence is okay, the method will return true, else the
 * method will return false.
 * @return boolean
 * @param orderStatusAB com.ibm.commerce.messaging.ejb.objects.OrderFulfillmentStatusAccessBean
 * @exception com.ibm.commerce.exception.ECException
 */
public boolean checkOrderStatusSequence(OrderFulfillmentStatusAccessBean orderStatusAB) throws com.ibm.commerce.exception.ECException 
{
	String methodName = "checkOrderStatusSequence";
	
	try {
		String orgSeqString = orderStatusAB.getSequenceNumber();
		Long newSeq = osProp.getLong(OrderFulfillmentStatusConstants.SEQUENCE_NUMBER, null);
		if (orgSeqString != null && newSeq != null) {
			Long orgSeq = Long.valueOf(orgSeqString);
			if (orgSeq.longValue() >= newSeq.longValue())
				return false;
		}
		Timestamp orgTs = orderStatusAB.getLastUpdateTime();
		String newTsString = osProp.getString(OrderFulfillmentStatusConstants.LAST_UPDATE_TIME, null);
		if (orgTs != null && newTsString != null) {
			Timestamp newTs = Timestamp.valueOf(newTsString);
			if (!newTs.after(orgTs)) {
				return false;
			}
		}
	}
	catch( javax.ejb.CreateException ex ) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), methodName,
						ECMessageHelper.generateMsgParms( ex.getMessage()), ex );
	}
	catch( javax.naming.NamingException ex ) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch( java.rmi.RemoteException ex ) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch (FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, this.getClass().getName(), methodName, 
			ECMessageHelper.generateMsgParms(ex.getMessage()), ex);

	} 
	return true;
}
/**
 * Creates a new entry in the ORDSTAT table based the the given information that is
 * received through the request properties.  ECException will be thrown if there
 * is any problems with the update.
 * @exception com.ibm.commerce.exception.ECException
 */
public void createOrderStatus() throws com.ibm.commerce.exception.ECException 
{
	String METHOD_NAME = "createOrderStatus";

	OrderFulfillmentStatusAccessBean orderStatusAB;
	try {
		orderStatusAB = new OrderFulfillmentStatusAccessBean(osProp);
		orderStatusAB.commitCopyHelper();
		osProp.put(OrderFulfillmentStatusConstants.ORDER_STATUS_ID, orderStatusAB.getOrderStatusId());
	} catch (CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), METHOD_NAME, ex);
	} catch (FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, this.getClass().getName(), METHOD_NAME, ex);
	} catch (NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, this.getClass().getName(), METHOD_NAME, ex);
	} catch (RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), METHOD_NAME, ex);
	}
	
}
/**
 * Performs a verification check after the command executes.
 * @return boolean
 * @param aRequestProperty com.ibm.commerce.datatype.TypedProperty
 * @exception ECException
 */
public boolean doPostProcess(TypedProperty aRequestProperty) throws ECException {
	return true;
}
/**
 * Performs a verification check before the command executes.
 * @return boolean
 * @param aRequestProperty com.ibm.commerce.datatype.TypedProperty
 * @exception ECException
 */
public boolean doPreProcess(TypedProperty aRequestProperty) throws ECException {
	return true;
}
/**
 * Determine whether this order is new or not.  If so, will add a new entry
 * to the ORDSTAT table providing a WCS Order does exist.  Else, an
 * existing entry in the ORDSTAT table will be updated providing the sequence found
 * in the request properties does not conflict with the sequence information found
 * in the ORDSTAT table for the entry to be updated.  Updating will depend on
 * where versioning equals "TRUE".  Versioning will update and version the entry
 * and a new entry will be created in the ORDSTAT table with the information found
 * in the request properties.  No versioning only consists of updating the entry.
 * After the order status is created or updated, the order items in the status
 * message will be updated. The method will return false or throw an ECException if there
 * is a problem with the given information or the update.  If there are no problems, the
 * method will return true.
 * @return boolean
 * @param aRequestProperty com.ibm.commerce.datatype.TypedProperty
 * @exception ECException
 */
public boolean doProcess(TypedProperty aRequestProperty) throws ECException 
{
	OrderFulfillmentStatusAccessBean abOS = getOrderStatus();
//	OrderFulfillmentStatusAccessBean abOS = null;
	if (abOS == null) {
		// It's new Order Status	
  		OrderAccessBean abOrder = getOrder();
  		if (abOrder == null) {	
			ECMessageLog.out(ECMessage._ERR_OS_ORDER_NOT_EXIST, this.getClass().getName(), "doProcess", iOrderId.toString());
			return false;
		}
  		createOrderStatus();
  		updateOrder(abOrder);
	} else {
		if (!checkOrderStatusSequence(abOS)) {
			ECMessageLog.out(ECMessage._ERR_OS_SEQUENCE, this.getClass().getName(), "doProcess", iOrderId.toString());
			return false;
		}
		String versioning = osProp.getString(OrderFulfillmentStatusConstants.VERSIONING, null);
		if (versioning != null && versioning.toUpperCase().equals("TRUE")) {
			updateOrderStatusVersion(abOS);
			createOrderStatus();
		} else {
			updateOrderStatus(abOS);
		}
	}
	processOrderStatusItem();
	processResponse();
 	return true;
}
/**
 * Get order.
 * Creation date: (7/7/00 5:58:03 PM)
 * @return com.ibm.commerce.order.objects.OrderAccessBean
 * @exception com.ibm.commerce.exception.ECException The exception description.
 */
private OrderAccessBean getOrder() throws com.ibm.commerce.exception.ECException 
{
	String methodName = "getOrder";
	
	try {
		OrderAccessBean abOrder = MiscCmd.findOrderForUpdate(iOrderId);
		return abOrder;
	} 
	catch( javax.ejb.CreateException ex ) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), methodName,
						ECMessageHelper.generateMsgParms( ex.getMessage()), ex );
	}
	catch( javax.naming.NamingException ex ) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch( java.rmi.RemoteException ex ) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	
}
/**
 * Get order status.
 * Creation date: (7/7/00 5:56:47 PM)
 * @return com.ibm.commerce.messaging.ejb.objects.OrderFulfillmentStatusAccessBean
 * @exception com.ibm.commerce.exception.ECException The exception description.
 */
private OrderFulfillmentStatusAccessBean getOrderStatus() throws com.ibm.commerce.exception.ECException 
{
	String methodName = "getOrderStatus";

	try {
		if (iMerchantOrderNumber != null) {
			try {
				OrderFulfillmentStatusAccessBean osAB = new OrderFulfillmentStatusAccessBean();
				osAB = osAB.findByMerchantOrderNumber(iMerchantOrderNumber);
				if (iOrderId == null) {
					iOrderId = osAB.getOrderId();
				} else {
					Long orderId = osAB.getOrderId();
					if (orderId != null && orderId.longValue() != iOrderId.longValue()) {
						throw new ECApplicationException(ECMessage._ERR_OS_ORDERID_MISMATCH, this.getClass().getName(), methodName);
					}
				}
				return osAB;
			} catch (FinderException e) {
				if (iOrderId == null) { 
					// can't find orderId by merchant order number
					throw new ECApplicationException(ECMessage._ERR_OS_ORDER_NOT_EXIST, this.getClass().getName(), methodName,
							ECMessageHelper.generateMsgParms(iMerchantOrderNumber));
				}
			}
		}
		if (iOrderId != null) {
			try {
		 		OrderFulfillmentStatusAccessBean osOrderAB = new OrderFulfillmentStatusAccessBean();
				osOrderAB = osOrderAB.findByOrderId(iOrderId);
				String existMerchantOrder = osOrderAB.getMerchantOrderNumber();
				if (iMerchantOrderNumber == null)
				    iMerchantOrderNumber = existMerchantOrder;
				else {
					if (existMerchantOrder != null && !existMerchantOrder.equals(iMerchantOrderNumber)) {
						throw new ECApplicationException(ECMessage._ERR_OS_ORDER_NUMBER_MISMATCH, this.getClass().getName(), methodName);
					}
				}
				return osOrderAB;
			} catch (FinderException ex) {
				return null;
			}
		
		}
		return null;
	} 
	catch( javax.ejb.CreateException ex ) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), methodName,
						ECMessageHelper.generateMsgParms( ex.getMessage()), ex );
	}
	catch( javax.naming.NamingException ex ) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch( java.rmi.RemoteException ex ) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
}
/**
 * This method returns the Organization unit reference numbers for all the resources accessed by this command
 * It will return null if no resource if being accessed by this command.
 *
 *	
 * @return Long[] - an array of all organization unit reference numbers
 */
public Long[] getResourceOwners() throws ECException {
	// get all resource owners for this command
	Long[] resowners = null;
	resowners = new Long[1];
	resowners[0] = new Long(ECConstants.EC_ACC_ALL_RESOURCES); // Access to all Owner's

	return resowners;
}
 public AccessVector getResources() throws ECException {
          final String methodName = "getResources";
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), methodName);
	//com.ibm.commerce.orderstatus.objects.OrderFulfillmentItemStatusBean osAB = null;

	try {
		
		if (iOrderId == null) {
			OrderFulfillmentStatusAccessBean osAB = new OrderFulfillmentStatusAccessBean();
			osAB = osAB.findByMerchantOrderNumber(iMerchantOrderNumber);
			if (iOrderId == null) {
				iOrderId = osAB.getOrderId();
			} else {
				Long orderId = osAB.getOrderId();
				if (orderId != null && orderId.longValue() != iOrderId.longValue()) {
					throw new ECApplicationException(ECMessage._ERR_OS_ORDERID_MISMATCH, this.getClass().getName(), methodName);
				}
			}
		}


		com.ibm.commerce.order.objects.OrderAccessBean oab = new com.ibm.commerce.order.objects.OrderAccessBean();
		oab.setInitKey_orderId(iOrderId.toString());
		oab.refreshCopyHelper();

		return new AccessVector(oab);
	} 
	catch( javax.ejb.CreateException ex ) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), methodName,
						ECMessageHelper.generateMsgParms( ex.getMessage()), ex );
	}
	catch( javax.naming.NamingException ex ) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch( java.rmi.RemoteException ex ) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch( javax.ejb.FinderException ex ) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, this.getClass().getName(), methodName,
						ECMessageHelper.generateMsgParms( ex.getMessage()), ex );
	}
}
/**
 * Indicates whether the command is ready to execute.
 * @return boolean
 */
public boolean isReadyToCallExecute()
{
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "isReadyToCallExecute");

		if (!super.isReadyToCallExecute())
			return false;

		if (iOrderId == null && iMerchantOrderNumber == null) {
			ECMessageLog.out(ECMessage._ERR_OS_PARAMS, this.getClass().getName(), "isReadyToCallExecute");
			return false;
		}

		ECTrace.exit(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "isReadyToCallExecute");

		return true;
}
/**
 * Performs a pre condition check on the request properties and if the 
 * pre condition check is ok, the business logic will be performed followed 
 * by a post condition check to ensure integrity is maintained. 
 * If either of the 3 steps fail, an ECException will be thrown.
 * @exception ECException
 */
public void performExecute() throws ECException
	{

		super.performExecute();
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "performExecute");

		if (!doPreProcess(requestProperties)) {
			throw new ECApplicationException(ECMessage._ERR_DO_PRE_PROCESS, this.getClass().getName(), "performExecute");
		}	

		if (!doProcess(requestProperties)) {
			throw new ECApplicationException(ECMessage._ERR_DO_PROCESS, this.getClass().getName(), "performExecute");
		}	
			
		if (!doPostProcess(requestProperties)) {
			throw new ECApplicationException(ECMessage._ERR_DO_POST_PROCESS, this.getClass().getName(), "performExecute");
		}
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "performExecute");

	}
/**
 * Process order status item.
 * Creation date: (7/11/00 4:39:46 PM)
 * @return boolean
 * @exception com.ibm.commerce.exception.ECException The exception description.
 */
private void processOrderStatusItem() throws com.ibm.commerce.exception.ECException 
{
	final String methodName = "processOrderStatusItem";

	if (osItems == null)
	    return;
	    
	for (Enumeration e = osItems.elements(); e.hasMoreElements();) {
		TypedProperty prop = (TypedProperty)e.nextElement();
		
		// defect 7784 - check order number at item level to ensure a match
		// NOTE: iOrderId should never be null because set my getOrdserStatus method
		if (iOrderId != null) {
			if (prop.get(OrderFulfillmentStatusConstants.ORDER_ID, null) == null) {
				// Order ID is not specified at item level, use header level
				prop.put(OrderFulfillmentStatusConstants.ORDER_ID, iOrderId.toString());
			}
			else if(! iOrderId.toString().equals(prop.getString(OrderFulfillmentStatusConstants.ORDER_ID, null))) {
				// Order ID at item level is not the same as header level
				throw new ECApplicationException(ECMessage._ERR_OS_ORDERID_MISMATCH, this.getClass().getName(), methodName);
			}
			// Order ID specified at item level and matches header level				
		}
		
		// defect 7784 - check order number at item level to ensure a match	
		if (iMerchantOrderNumber != null) {			
			if (prop.get(OrderFulfillmentStatusConstants.MERCHANT_ORDER_NUMBER, null) == null) {
				// Backend ID is not specified at item level, use header level
				prop.put(OrderFulfillmentStatusConstants.MERCHANT_ORDER_NUMBER, iMerchantOrderNumber);
			}
			else if(! iMerchantOrderNumber.equals(prop.getString(OrderFulfillmentStatusConstants.MERCHANT_ORDER_NUMBER, null))) {
				// Backend ID at item level is not the same as header level
				throw new ECApplicationException(ECMessage._ERR_OS_ORDER_NUMBER_MISMATCH, this.getClass().getName(), methodName);
			}
			// Backend ID specified at item level and matches header level							
		}

		try	{
			OrderItemStatusCmd api = (OrderItemStatusCmd)CommandFactory.createCommand(OrderItemStatusCmd.NAME, ECConstants.EC_NO_STOREID);
			api.setOrderItemStatusProperties(prop);
			api.setCommandContext( this.getCommandContext() );
			api.execute();
 		} 
	   	catch ( ECException ex ) {
			ECTrace.trace( ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), methodName, 
				"Caught an ECException: " + ex.toString() );
				// This exception would be thrown by the API.  Just re-throw.
			throw ex;
	  	}
		catch ( CommandException ex ) 		
		{
			// A CommandException may wrap an ECException (including an ECApplicationException)
			Throwable exc = ex.getException();
			if ( exc != null && exc instanceof ECException )
			{
				ECException wrappedExc = (ECException) exc;
				ECTrace.trace( ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), methodName, 
					"Caught a CommandException with an ECException: " + wrappedExc.toString() +  
					".\n Detail Message: " + wrappedExc.getMessage() );
			    throw wrappedExc;			// Re-throw the ECException.
			}
			else	// Caught a CommandException which is not an ECExeption, re-throw as ECSystemException
			{
				throw new ECSystemException( 
							ECMessage._ERR_COMMAND_EXCEPTION, this.getClass().getName(), methodName, 
							ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
			}
		}
	}
}
/**
 * Process response.
 * Creation date: (7/11/00 4:39:46 PM)
 * @return boolean
 * @exception com.ibm.commerce.exception.ECException The exception description.
 */
private void processResponse() throws com.ibm.commerce.exception.ECException 
{
	final String methodName = "processResponse";
/*  // for unit test
	OrderStatusNotifySendCmdImpl osn= new OrderStatusNotifySendCmdImpl();
	osn.setCommandContext(getCommandContext());
	osn.setStoreID(getStoreId());
	osn.setOrderStatusProperties(osProp);
	osn.performExecute();
*/
	try	{
		OrderStatusNotifySendCmd api = (OrderStatusNotifySendCmd)CommandFactory.createCommand(OrderStatusNotifySendCmd.NAME, ECConstants.EC_NO_STOREID);
		api.setOrderStatusProperties(osProp);
		api.setStoreID(getStoreId());
		api.setCommandContext( this.getCommandContext() );
		api.execute();
 	} 
   	catch ( ECException ex ) {
		ECTrace.trace( ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), methodName, 
			"Caught an ECException: " + ex.toString() );
			// This exception would be thrown by the API.  Just re-throw.
		throw ex;
  	}
	catch ( CommandException ex ) 		
	{
		// A CommandException may wrap an ECException (including an ECApplicationException)
		Throwable exc = ex.getException();
		if ( exc != null && exc instanceof ECException )
		{
			ECException wrappedExc = (ECException) exc;
			ECTrace.trace( ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), methodName, 
				"Caught a CommandException with an ECException: " + wrappedExc.toString() +  
				".\n Detail Message: " + wrappedExc.getMessage() );
		    throw wrappedExc;			// Re-throw the ECException.
		}
		else	// Caught a CommandException which is not an ECExeption, re-throw as ECSystemException
		{
			throw new ECSystemException( 
						ECMessage._ERR_COMMAND_EXCEPTION, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
		}
	}

}
/**
 * Fills the OrderFulfillmentStatusAccessBean object with the values that are found
 * in the request properties.  If values found in the request properties conflict with
 * the allowed values for the OrderFulfillmentStatusAccessBean, an ECException is
 * thrown.
 * @param ab com.ibm.commerce.messaging.ejb.objects.OrderFulfillmentStatusAccessBean
 * @param aProp com.ibm.commerce.datatype.TypedProperty
 * @exception com.ibm.commerce.exception.ECException
 */
public void propertySet(OrderFulfillmentStatusAccessBean ab, TypedProperty aProp) throws ECException
{
	final String methodName = "propertySet";
	String value = null;
	String key = null;

	try {
	 	key = OrderFulfillmentStatusConstants.ORDER_ID;
		if ((value = aProp.getString(key, null)) != null) {
			if (ab.getOrderId() == null) 
				ab.setOrderId(new Long(value));
		}
		key = OrderFulfillmentStatusConstants.MERCHANT_ORDER_NUMBER;
		if (ab.getMerchantOrderNumber() == null)
			ab.setMerchantOrderNumber(aProp.getString(key, null));

		key = OrderFulfillmentStatusConstants.CURRENCY;
		if ((value = aProp.getString(key, null)) != null)
			ab.setCurrency(value);
		key = OrderFulfillmentStatusConstants.PRICE_TOTAL;
		if ((value = aProp.getString(key, null)) != null)
		    ab.setPriceTotal(new BigDecimal(value));
		key = OrderFulfillmentStatusConstants.TAX_TOTAL;
		if ((value = aProp.getString(key, null)) != null)
			ab.setTaxTotal(new BigDecimal(value));
		key = OrderFulfillmentStatusConstants.SHIPPING_TOTAL;
		if ((value = aProp.getString(key, null)) != null)
			ab.setShippingTotal(new BigDecimal(value));
		key = OrderFulfillmentStatusConstants.SHIPPING_TAX_TOTAL;
		if ((value = aProp.getString(key, null)) != null)
			ab.setShippingTaxTotal(new BigDecimal(value));
		key = OrderFulfillmentStatusConstants.ORDER_STATUS;
		if ((value = aProp.getString(key, null)) != null)
			ab.setOrderStatus(value);
		key = OrderFulfillmentStatusConstants.PLACE_DATE_TIME;		
		if ((value = aProp.getString(key, null)) != null)
			ab.setPlaceDateTime(Timestamp.valueOf(value));
		key = OrderFulfillmentStatusConstants.REQUEST_SHIP_DATE_TIME;
		if ((value = aProp.getString(key, null)) != null)
			ab.setRequestShipDateTime(Timestamp.valueOf(value));
		key = OrderFulfillmentStatusConstants.SCHEDULE_SHIP_DATE_TIME;
		if ((value = aProp.getString(key, null)) != null)
			ab.setScheduleShipDateTime(Timestamp.valueOf(value));
		key = OrderFulfillmentStatusConstants.ACTUAL_SHIP_DATE_TIME;
		if ((value = aProp.getString(key, null)) != null)
			ab.setActualShipDateTime(Timestamp.valueOf(value));
		key = OrderFulfillmentStatusConstants.INVOICE_DATE_TIME;
		if ((value = aProp.getString(key, null)) != null)
			ab.setInvoiceDateTime(Timestamp.valueOf(value));
		key = OrderFulfillmentStatusConstants.INVOICE_VALUE;
		if ((value = aProp.getString(key, null)) != null)
			ab.setInvoiceValue(new BigDecimal(value));
		key = OrderFulfillmentStatusConstants.SHIP_CONDITION;
		if ((value = aProp.getString(key, null)) != null)
			ab.setShipCondition(value);
		key = OrderFulfillmentStatusConstants.SHIPPING_MODE_FLAG;
		if ((value = aProp.getString(key, null)) != null)
			ab.setShippingModeFlag(value);
		key = OrderFulfillmentStatusConstants.LAST_UPDATE_TIME;
		if ((value = aProp.getString(key, null)) != null)
			ab.setLastUpdateTime(Timestamp.valueOf(value));
		key = OrderFulfillmentStatusConstants.SEQUENCE_NUMBER;
		if ((value = aProp.getString(key, null)) != null)
			ab.setSequenceNumber(value);
		key = OrderFulfillmentStatusConstants.COMMENT;
		if ((value = aProp.getString(key, null)) != null)
			ab.setComment(value);
		key = OrderFulfillmentStatusConstants.FIELD1;
		if ((value = aProp.getString(key, null)) != null)
			ab.setField1(new Integer(new Double(value).intValue()));
		key = OrderFulfillmentStatusConstants.FIELD2;
		if ((value = aProp.getString(key, null)) != null)
			ab.setField2(new BigDecimal(value));
		key = OrderFulfillmentStatusConstants.FIELD3;
		if ((value = aProp.getString(key, null)) != null)
			ab.setField3(value);
	}
	catch (NumberFormatException e) {
		ECMessageLog.out(ECMessage._ERR_OS_INVALID_NUMERIC, this.getClass().getName(), methodName, value, key);
		throw new InvalidParameterValueException(key, value);
	}
	catch (IllegalArgumentException e) {
		ECMessageLog.out(ECMessage._ERR_OS_INVALID_TIMESTAMP, this.getClass().getName(), methodName, value, key);
		throw new InvalidParameterValueException(key, value);
	}
	catch( javax.ejb.CreateException ex ) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), methodName,
						ECMessageHelper.generateMsgParms( ex.getMessage()), ex );
	}
	catch( javax.naming.NamingException ex ) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch( java.rmi.RemoteException ex ) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch (FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, this.getClass().getName(), methodName, 
			ECMessageHelper.generateMsgParms(ex.getMessage()), ex);

	} 

	
}
/**
 * An Empty method.
 */
public void reset () {}
/**
 * Binds the parameters received by the request properties to variables that are used
 * for determining the order to updated.  Also, a validation check is performed on
 * some of the given parameters and if this check fails, an ECException is thrown.
 * @param aRequestProperty com.ibm.commerce.datatype.TypedProperty
 * @exception ECException
 */
public void setRequestProperties(TypedProperty aRequestProperties) throws ECException
{
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "setRequestProperties");

		requestProperties = aRequestProperties;
		osProp = aRequestProperties;
		
		iOrderId = requestProperties.getLong(OrderFulfillmentStatusConstants.ORDER_ID, null);
		// verify that order id was specified and of type long.
		if(iOrderId == null) {
			String testOrderId = requestProperties.getString(OrderFulfillmentStatusConstants.ORDER_ID, null);
			if(testOrderId != null && testOrderId.trim().length() > 0) {
				throw new ECApplicationException(ECMessage._ERR_INVALID_ORDER_REFNUM, this.getClass().getName(), "setRequestProperties");
			}
		}
		
		iMerchantOrderNumber = requestProperties.getString(OrderFulfillmentStatusConstants.MERCHANT_ORDER_NUMBER, null);

		osItems = (Vector) requestProperties.get(OrderFulfillmentStatusConstants.ITEMS_VECTOR, null);
				
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "setRequestProperties");
}
/**
 * Updates the ORDSTAT table with the information given by the request properties.
 * @param abOrder com.ibm.commerce.order.objects.OrderAccessBean
 * @exception com.ibm.commerce.exception.ECException
 */
public void updateOrder(OrderAccessBean abOrder) throws com.ibm.commerce.exception.ECException 
{
	String METHOD_NAME = "updateOrder";
	// If the order does not exist, or is neither pending nor cancelled, the command completes
		// successfully but does nothing.		
   try {
	   if (abOrder != null)
		{
			//Set Fulfillment status.
//			abOrder.setLock(OrderSqlHelper.ORDER_UNLOCKED);
//				abOrder.setStatus(OrderSqlHelper.ORDER_FUFILLMENT);
			abOrder.setStatus("G");
			abOrder.setPlaceOrderTime(new Timestamp(System.currentTimeMillis()));
			abOrder.commitCopyHelper();

			// Update the orderitems so that they too are marked as having been cancelled
			OrderItemAccessBean abOrderItem = new OrderItemAccessBean();
			Enumeration enOrderItems = abOrderItem.findByOrderForUpdate(abOrder.getOrderIdInEJBType());
			if (enOrderItems != null && enOrderItems.hasMoreElements()) {
				while (enOrderItems.hasMoreElements())
				{
					abOrderItem = (OrderItemAccessBean) enOrderItems.nextElement();
//					abOrderItem.setStatus(OrderSqlHelper.ORDER_FUFILLMENT);
					abOrderItem.setStatus("G");
				}
				abOrderItem.commitCopyHelper();
			}
		}
	}
	catch (RemoteException e)
	{
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), METHOD_NAME, ECMessageHelper.generateMsgParms(e.getMessage()), e);
	}
	catch (CreateException e)
	{
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), METHOD_NAME, ECMessageHelper.generateMsgParms(e.getMessage()), e);
	}
	catch (NamingException e)
	{
		throw new ECSystemException(ECMessage._ERR_GENERIC, this.getClass().getName(), METHOD_NAME, ECMessageHelper.generateMsgParms(e.getMessage()), e);
	}
	catch (FinderException e)
	{
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, this.getClass().getName(), METHOD_NAME, ECMessageHelper.generateMsgParms(e.getMessage()), e);

	} 

}
/**
 * Update order status.
 * Creation date: (7/10/00 11:45:10 AM)
 * @param orderStatusAB com.ibm.commerce.messaging.ejb.objects.OrderFulfillmentStatusAccessBean
 * @exception com.ibm.commerce.exception.ECException The exception description.
 */
private void updateOrderStatus(OrderFulfillmentStatusAccessBean orderStatusAB) throws com.ibm.commerce.exception.ECException 
{
	String methodName = "updateOrderStatus";
	
	try {
//		orderStatusAB.propertySet(osProp);
		propertySet(orderStatusAB, osProp);
		orderStatusAB.commitCopyHelper();
		osProp.put(OrderFulfillmentStatusConstants.ORDER_STATUS_ID, orderStatusAB.getOrderStatusId());
	}
	catch( javax.ejb.CreateException ex ) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), methodName,
						ECMessageHelper.generateMsgParms( ex.getMessage()), ex );
	}
	catch( javax.naming.NamingException ex ) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch( java.rmi.RemoteException ex ) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch (FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, this.getClass().getName(), methodName, 
			ECMessageHelper.generateMsgParms(ex.getMessage()), ex);

	} 
}
/**
 * Update order status version.
 * Creation date: (7/10/00 12:18:42 PM)
 * @param orderStatusAB com.ibm.commerce.messaging.ejb.objects.OrderFulfillmentStatusAccessBean
 * @exception com.ibm.commerce.exception.ECException The exception description.
 */
private void updateOrderStatusVersion(OrderFulfillmentStatusAccessBean orderStatusAB) throws com.ibm.commerce.exception.ECException 
{
	String methodName = "updateOrderStatusVersion";
	
	try {
		OrderFulfillmentStatusAccessBean maxAB = new OrderFulfillmentStatusAccessBean();
		maxAB = maxAB.findByMaxVersion(iOrderId, iOrderId);
		orderStatusAB.setVersioning(new Integer(maxAB.getVersioning().intValue() + 1));
		orderStatusAB.commitCopyHelper();
	}
	catch( javax.ejb.CreateException ex ) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), methodName,
						ECMessageHelper.generateMsgParms( ex.getMessage()), ex );
	}
	catch( javax.naming.NamingException ex ) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch( java.rmi.RemoteException ex ) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch (FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, this.getClass().getName(), methodName, 
			ECMessageHelper.generateMsgParms(ex.getMessage()), ex);

	} 
	

}
}
