//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.returns.commands.*;

/**
 * Command used for calculating overall status of return mechandise authorization (RMA).
 * If any RMA item is not approved (RMAITEM.STATUS='APP') or manually approved
 * (RMAITEM.STATUS='MAN'), then the RMA is set to pending (RMA.STATUS='PND').
 * Otherwise, it is set to approved (RMA.STATUS='APP').
 */
import java.sql.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.command.*;
import java.util.*;
import javax.naming.*;
import java.rmi.*;
import javax.ejb.*;
import com.ibm.commerce.ordermanagement.objects.*;

public class CalculateRMAApprovalCmdImpl extends com.ibm.commerce.command.TaskCommandImpl implements CalculateRMAApprovalCmd {
	private static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	private final static java.lang.String className = CalculateRMAApprovalCmdImpl.class.getName();
	private RMAAccessBean rmaAB = null;
	private Vector vRMAItemAB = null;
	boolean RMAapproved = true;
/**
 * CalculateRMAApprovalCmdImpl constructor.
 */
public CalculateRMAApprovalCmdImpl() {
	super();
}
/**	
 * Loop through each RMAitem and determine the rolled up status of RMA.
 */
public void calculateRMAApproval()
	throws FinderException, RemoteException, NamingException, CreateException, ECSystemException {

	final String methodName = "calculateRMAApproval";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);
	ECMessageLog.out(ECMessage._STA_WRITING_ORDERITEM, className, methodName);

	/* Loop through all the RMAItems */
	for (int i = 0; i < vRMAItemAB.size(); i++) {
		RMAItemAccessBean abRMAItem = (RMAItemAccessBean) vRMAItemAB.elementAt(i);
		/* Right now not equal to manual or system approved == pending, but 
		 I see this as likely to change, eg more RMA item statuses probable to be added,
		 so I'm explicitly using the 2 forms of approved */
		if (!abRMAItem.getStatus().equals(States.cItemAutoApproved)
				&& !abRMAItem.getStatus().equals(States.cItemManualApproved)) {
			RMAapproved = false;
		}
	}

	if (RMAapproved)
		rmaAB.setStatus(States.cRMAApproved);
	else
		rmaAB.setStatus(States.cRMAPending);
	Timestamp now = com.ibm.commerce.utils.TimestampHelper.systemCurrentTimestamp();
	rmaAB.setLastUpdate(now);

}
/**
 * Return RMA access bean for RMA being evaluated.
 * @return RMA access bean
 */
private com.ibm.commerce.ordermanagement.objects.RMAAccessBean getRMAAB() {
	return rmaAB;
}
/**	Whether RMA was approved
 * @return true if approved; otherwise false.
 */
public boolean isRMAApproved() {

	final String methodName = "isRMAApproved";

	return RMAapproved;

}
/**
 * Execute the command.
 */
public void performExecute() throws ECException {
	final String methodName = "performExecute";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);

	super.performExecute();

	try {
		calculateRMAApproval();
	}
	catch (NamingException e) {
		throw new ECSystemException(ECMessage._ERR_GENERIC,className,methodName,
																ECMessageHelper.generateMsgParms(e.getMessage()),e);
	}
	catch (RemoteException e) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION,className,methodName,
																ECMessageHelper.generateMsgParms(e.getMessage()),	e);
	}
	catch (CreateException e) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION,className,methodName,
																ECMessageHelper.generateMsgParms(e.getMessage()),e);
	}
	catch (FinderException e) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION,className,methodName,
																ECMessageHelper.generateMsgParms(e.getMessage()),e);
	}

}
public void reset() {
	rmaAB = null;
	vRMAItemAB = null;
	RMAapproved = true;
}
/**
 * Set the RMA being accessed.
 * @param rmaAB the RMA access bean.
 */
public void setRMAAB(RMAAccessBean rmaAB) {
	this.rmaAB = rmaAB;
}
/**
 * Set the RMA items to evaluate for RMA approval
 * @param vRMAItemAB vector of RMA item access beans.
 */
public void setRMAItemABVector(Vector vRMAItemAB) {
	this.vRMAItemAB = vRMAItemAB;
}
public void validateParameters() throws ECException {
	final String methodName = "validateParameters";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);

	super.validateParameters();

	/* check the RMA */
	if (rmaAB == null) {
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,className,methodName,
																		 new Object[] { ReturnConstants.EC_RMA_ID });
	}

	/* check the enumeration of RMA items */
	if (vRMAItemAB == null) {
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,className,methodName,
																		 new Object[] { ReturnConstants.EC_RMA_ITEM_ID });
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);
}
}
